import java.util.*;
import java.io.*;

/** Demonstrates a class loader written for Java 1.1.
This is also compatible with Java 1.2.
*/
public class CustomClassLoader extends java.lang.ClassLoader
{	
	/** Reads the class data from a class file. Replace this routine to read 
	the class data from a source other than a file.
	*/
    public byte[] readClassFile(String className)
    {
		byte[]				inputBytes = null;
		int					bytesRead = 0;
		File				classFile = null;
		FileInputStream		fileStream = null;
		String				fileName;
		String				pathName;
		
		/* Support package names by converting the class name to 
		a file name by replacing the "." in the package name with a 
		path separator. This only applies because the class file is being 
		loaded from a file.
		*/
		fileName = className.replace('.', File.separatorChar);
		pathName = ".\\TestClass\\" + fileName + ".class";
		
		/* Read the class file into a buffer in one shot */
		try {
			classFile = new File(pathName);
			fileStream = new FileInputStream(classFile);
			
			/* Create a buffer big enough to hold the entire class file. */
			bytesRead = (int)classFile.length();
			inputBytes = new byte[bytesRead];
			
			/* Read the class file into the buffer */
			try {
				bytesRead = fileStream.read(inputBytes);
			}
			catch (IOException e) {
				System.out.println("Exception trying to read the class file");
			}
		}
		catch (FileNotFoundException e) {
			System.out.println("File " + classFile.toString() + " doesn't exist");	
		}
		return inputBytes;
	}
	
	
	/** A constructor with a null parameter is required.
	Even if it does nothing.
	*/
	public CustomClassLoader() {
		
	}
		
	
    public synchronized Class loadClass(String name, boolean resolve) throws ClassNotFoundException
    {
    	Class		newClass;
    	byte[]		classData;
    	String		className;
    	
    	newClass = findLoadedClass(name);	// check the cache for the class
        if (newClass == null) {
    		try {
    			newClass = findSystemClass(name);	// ask the system loader to load the class
    			System.out.println("Class " + name + " loaded by the system loader");
    		}catch (ClassNotFoundException e) {
    			/* Class was not found in the cache and couldn't be loaded by the system. 
    			Load the class ourselves.
    			*/
    			classData = readClassFile(name);
        		if (classData == null) {
        			throw e;	// rethrow the ClassNotFoundException
        		}
        		newClass = this.defineClass(name, classData, 0, classData.length);        			
        		className = newClass.getName();
        		System.out.println("Class " + className + " loaded by CustomClassLoader");
    		}
        }
        else {
        	System.out.println("Class " + name + " already loaded by custom class loader");
        }
		if (resolve) {
			this.resolveClass(newClass);
		}
		return newClass;
    }
        
}	//CustomClassLoader
