import java.awt.*;
import java.lang.reflect.*;

import DeviceControls;		// interface for the DeviceController class


/** Displays a frame with a single button for testing the class loaders. 
Clicking the "Load Classes" button toggles between the Java 1.1 and Java 1.2 
compatible class loaders.
*/
public class Frame1 extends Frame
{
	DeviceConnection	mDevice;
	CustomClassLoader	mClassLoader;
	CustomLoader2		mClassLoader2;
	int					mLoaderCount = 0;
	
	//{{DECLARE_CONTROLS
	java.awt.Button LoadButton = new java.awt.Button();
	//}}
	
	
	//{{DECLARE_MENUS
	//}}
	
	static public void main(String args[])
	{
		Frame1		mainFrame;
			
		//Create a new instance of our application's frame, and make it visible.
		mainFrame = new Frame1();
    	mainFrame.setVisible(true);
	}
	
	public Frame1()
	{
		// This code is automatically generated by Visual Cafe when you add
		// components to the visual environment. It instantiates and initializes
		// the components. To modify the code, only use code syntax that matches
		// what Visual Cafe can generate, or Visual Cafe may be unable to back
		// parse your Java file into its visual environment.
		
		//{{INIT_CONTROLS
		setLayout(null);
		setSize(207,143);
		setVisible(false);
		setLocation(50, 50);
		LoadButton.setLabel("Load Classes");
		add(LoadButton);
		LoadButton.setBackground(java.awt.Color.lightGray);
		LoadButton.setBounds(56,97,95,40);
		setTitle("Class Loader Test");
		//}}
		
		//{{INIT_MENUS
		//}}
		
		//{{REGISTER_LISTENERS
		SymWindow aSymWindow = new SymWindow();
		this.addWindowListener(aSymWindow);
		SymAction lSymAction = new SymAction();
		LoadButton.addActionListener(lSymAction);
		//}}
		
		/* Instantiate the class loaders only once during the course of the application. */
		mClassLoader = new CustomClassLoader();
		mClassLoader2 = new CustomLoader2();
	}
			
	class SymWindow extends java.awt.event.WindowAdapter
	{
		public void windowClosing(java.awt.event.WindowEvent event)
		{
			Object object = event.getSource();
			if (object == Frame1.this){
				dispose();
				System.exit(0);
			}
		}
	}
	
	class SymAction implements java.awt.event.ActionListener
	{
		public void actionPerformed(java.awt.event.ActionEvent event)
		{
			Object object = event.getSource();
			if (object == LoadButton)
				LoadButton_ActionPerformed(event);
		}
	}

	void LoadButton_ActionPerformed(java.awt.event.ActionEvent event)
	{
		Class				newDeviceClass;
		Class				controllerClass;
		DeviceControls		deviceController;
		Canvas				testClass;
		
		/* Class names must include the package */
		String				className = "mypackage.SimpleSquare";
		String				className2 = "mypackage.DeviceController";
		
		System.out.println("-----------------------------------------------------------");
		
		/* Test if the classes can be loaded by the system class loader.  
		This will always fail because the system class loader has no 
		record of our classes. 
		*/
		try {
			Class	test = Class.forName(className);
		}
		catch (ClassNotFoundException e) {
			System.out.println( e.toString() );
		}
		try {
			Class	test = Class.forName(className2);
		}
		catch (ClassNotFoundException e) {
			System.out.println( e.toString() );
		}
		
		try {
			if (mLoaderCount == 0){		// use CustomeClassLoader to load the classes
				newDeviceClass = mClassLoader.loadClass(className);
				controllerClass = mClassLoader.loadClass(className2);
				mLoaderCount = 1;
			}
			else {	// Use CustomLoader2 to load the classes
				newDeviceClass = mClassLoader2.loadClass(className);
				controllerClass = mClassLoader2.loadClass(className2);
				mLoaderCount = 0;
			}
			try {
				/* Use a parent class to execute a class loaded by 
				the custom class loader.
				Note the cast of the new object to the parent class Canvas.
				*/
				testClass = (Canvas)newDeviceClass.newInstance();
				this.add(testClass);
					
				/* Use an interface as a wrapper for a class loaded by 
				the custom class loader.
				Note the cast of the new object to the interface DeviceControls
				*/
				deviceController = (DeviceControls)controllerClass.newInstance();
				deviceController.onOff(true);
				int		battery = deviceController.getBatteryPower();
				System.out.println("Battery Power = " + battery);
					
				/* Use reflection to call the methods in the controllerClass */
				useReflection(controllerClass);
			}
			catch(InstantiationException e){ 
			}
			catch(IllegalAccessException e){ 
			}
		}
		catch (ClassNotFoundException e) {
			System.out.println("Rats! Couldn't load class " + className + " because it wasn't found");
		}
	}
	
    /** Demonstrate using refection to operate the device controller class.
    */
    private void useReflection(Class loadedClass)
	{
		Object		controllerObject;
		String		className;
		Method		onOffMethod;
		Method		getBatteryMethod;
		
		className = loadedClass.getName();
		if ( className.equals("mypackage.DeviceController") ) {
			/* For brevity, I'll assume the names of the methods once I know 
			the class name. If you really wanted, you could use reflection for everything.
			*/
			try {
				System.out.println("Calling onOff() and getBatterPower() using reflection");
				controllerObject = loadedClass.newInstance();
				
				try {
					/* Call the onOff() method */
					onOffMethod = loadedClass.getMethod( "onOff", new Class[]{boolean.class} );
					onOffMethod.invoke( controllerObject, new Object[]{new Boolean(true)} );
					
					/* Call the getBatteryPower() method */
					getBatteryMethod = loadedClass.getMethod( "getBatteryPower", null );
					getBatteryMethod.invoke( controllerObject, null );
				}
				catch(InvocationTargetException e){
				}
				catch(NoSuchMethodException e) {
					System.out.println(e.toString());
				}
				catch(SecurityException e){
					System.out.println(e.toString());
				}
			}
			catch (InstantiationException e) {
			}
			catch (IllegalAccessException e) {
			}
		}
    }
	
}

