/*
Copyright (C) Cedilla Software Services Ltd. 1999. License is granted to use this software subject to
the following conditions:-

Preamble:

The intent of this document is to state the conditions under which a Package may be copied, 
such that the Copyright Holder maintains some semblance of artistic control over the development 
of the package, while giving the users of the package the right to use and distribute the 
Package in a more-or-less customary fashion, plus the right to make reasonable modifications.

Definitions:

"Package" refers to the collection of files distributed by the Copyright Holder, and 
derivatives of that collection of files created through textual modification.

"Standard Version" refers to such a Package if it has not been modified, or has been modified 
in accordance with the wishes of the Copyright Holder.

"Copyright Holder" is whoever is named in the copyright or copyrights for the package.

"You" is you, if you're thinking about copying or distributing this Package.

"Reasonable copying fee" is whatever you can justify on the basis of media cost, duplication 
charges, time of people involved, and so on. (You will not be required to  justify it to 
the Copyright Holder, but only to the computing community at large as a market that must bear the fee.)

"Freely Available" means that no fee is charged for the item itself, though there may be 
fees involved in handling the item. It also means that recipients of the item may redistribute 
it under the same conditions they received it.

1. You may make and give away verbatim copies of the source form of the Standard Version of this
Package without restriction, provided that you duplicate all of the original copyright 
notices and associated disclaimers.

2. You may apply bug fixes, portability fixes and other modifications derived from the Public 
Domain or from the Copyright Holder. A Package modified in such a way shall still be considered 
the Standard Version.

3. You may otherwise modify your copy of this Package in any way, provided that you 
insert a prominent notice in each changed file stating how and when you changed that file, and provided 
that you do at least ONE of the following:

a) place your modifications in the Public Domain or otherwise make them Freely Available, 
such as by posting said modifications to Usenet or an equivalent medium, or placing the 
modifications on a major archive site such as ftp.uu.net, or by allowing the 
Copyright Holder to include your modifications in the Standard Version of the Package.

b) use the modified Package only within your corporation or organization.

c) rename any non-standard executables so the names do not conflict with 
standard executables, which must also be provided, and provide a separate manual page for 
each non-standard executable that clearly documents how it differs from the Standard Version.

d) make other distribution arrangements with the Copyright Holder.

4. You may distribute the programs of this Package in object code or executable 
form, provided that you do at least ONE of the following:

a) distribute a Standard Version of the executables and library files, together with 
instructions (in the manual page or equivalent) on where to get the Standard Version.

b) accompany the distribution with the machine-readable source of the Package
with your modifications.

c) accompany any non-standard executables with their corresponding Standard Version 
executables, giving the non-standard executables non-standard names, and clearly documenting 
the differences in manual pages (or equivalent), together with instructions on where to 
get the Standard Version.

d) make other distribution arrangements with the Copyright Holder.

5. You may charge a reasonable copying fee for any distribution of this Package. You may 
charge any fee you choose for support of this Package. You may not charge a fee for
this Package itself. However, you may distribute this Package in aggregate with other 
(possibly commercial) programs as part of a larger (possibly commercial) software
distribution provided that you do not advertise this Package as a product of your own.

6. The scripts and library files supplied as input to or produced as output from the 
programs of this Package do not automatically fall under the copyright of this Package, 
but belong to whomever generated them, and may be sold commercially, and may be 
aggregated with this Package.

7. C, Java or perl subroutines supplied by you and linked into this Package shall not be 
considered part of this Package.

8. The name of the Copyright Holder may not be used to endorse or promote products 
derived from this software without specific prior written permission.

9. THIS PACKAGE IS PROVIDED "AS IS" AND WITHOUT ANY EXPRESS OR IMPLIED WARRANTIES,
INCLUDING, WITHOUT LIMITATION, THE IMPLIED WARRANTIES OF MERCHANTIBILITY AND FITNESS
FOR A PARTICULAR PURPOSE.

The End

Acknowledgments:
*/
import java.awt.*;
import java.awt.event.*;
import java.util.*;
import javax.swing.*;

public class OptsParser implements Map {
    //Static Final Members//////////////////////////////////////////////////////
    private static final String HELP = "?";

    //Member Variables//////////////////////////////////////////////////////////
    private String _usage;
    private String[] _args;
    private HashMap _defs = new HashMap();
    private HashMap _vals = new HashMap();

    //Constructors//////////////////////////////////////////////////////////////
    public OptsParser(String usage) {
        _usage = usage;
    }

    //Public Methods////////////////////////////////////////////////////////////
    public void addDefinition(String name, String dflt, String message) {
        String[] def = new String[2];
        def[0] = dflt;
        def[1] = message;
        _defs.put(name, def);
    }

    public void parse(String[] args) {
        _args = new String[args.length];
        ArrayList nonOptions = new ArrayList();
        String lastOption = null;

        //First of all check for a help request. If it
        //present, print out all of the messages and exit. At
        //the same time, build a list of values

        boolean foundHelp = false;
        for (int i = 0; i < args.length; i++) {
            _args[i] = args[i];

            if (args[i].startsWith("-")) {
                if (lastOption != null) {
                    _vals.put(lastOption, toStringArray(nonOptions));
                    nonOptions.clear();
                }

                lastOption = args[i].substring(1);

                if (args[i].equals("-" + HELP)) {
                    foundHelp = true;
                    break;
                }
            } else {
                nonOptions.add(args[i]);
            }
        }

        if (foundHelp) {
            printAllMessages();
            System.exit(0);
        } else if (lastOption != null) {
            //System.out.println("Adding " + lastOption);
            _vals.put(lastOption, toStringArray(nonOptions));
        }

        //Secondly go through the set of options and check to see
        //they exit. If not print all messages and exit.

        boolean badOption = false;
        //System.out.println(vals.keySet());
        Object[] options = _vals.keySet().toArray();

        for (int i = 0; i < options.length; i++) {
            String option = (String) options[i];
            if (!_defs.containsKey(option)) {
                badOption = true;
                System.out.println("Unrecognised option: -" + option);
                break;
            }
        }

        if (badOption) {
            printAllMessages();
            System.exit(1);
        }

        //Thirdly, add any default values that have not been
        //entered by the user

        addRemainingDefaults();
    }

    public void printAllMessages() {
        Object[] keys = _defs.keySet().toArray();

        System.out.println(_usage);
        System.out.println("where options include:");

        for (int i = 0; i < keys.length; i++) {
            String[] defVals = (String[]) _defs.get(keys[i]);

            if (defVals[0] == null) {
                System.out.println("-" + keys[i] + " " + defVals[1]);
            } else {
                System.out.println("-" + keys[i] + " " + defVals[1] + " Defaults to " + defVals[0] + ".");
            }
        }

        System.out.println("-" + HELP + " Prints this message");
    }

    public String[] last(int n) {
        String[] out = null;

        if (n <= _args.length) {
            int diff = _args.length - n;

            out = new String[n];

            for (int i = 0; i < n; i++) {
                out[i] = _args[diff + i];
            }
        }

        return out;
    }

    public String[] getOption(String name) {
        return (String[])_vals.get(name);
    }

    //Methods needed to implement Map///////////////////////////////////////////
    public void clear() {
        _vals.clear();
    }

    public boolean containsKey(Object key) {
        return _vals.containsKey(key);
    }

    public boolean containsValue(Object value) {
        return _vals.containsValue(value);
    }

    public Set entrySet() {
        return _vals.entrySet();
    }

    public Object get(Object o) {
        return _vals.get(o);
    }

    public boolean isEmpty() {
        return _vals.isEmpty();
    }

    public Set keySet() {
        return _vals.keySet();
    }

    public Object put(Object key, Object val) {
        return _vals.put(key, val);
    }

    public void putAll(Map t) {
        _vals.putAll(t);
    }

    public Object remove(Object key) {
        return _vals.remove(key);
    }

    public int size() {
        return _vals.size();
    }

    public Collection values() {
        return _vals.values();
    }

    //Private Methods///////////////////////////////////////////////////////////
    private String[] toStringArray(ArrayList a) {
        String[] arr = new String[a.size()];

        for (int i = 0; i < a.size(); i++) {
            arr[i] = (String)a.get(i);
        }

        return arr;
    }

    /**
     * Add any non null default values from
     * the definitions where the option is not already
     * in the values.
     */
    private void addRemainingDefaults() {
     Object[] defKeys = _defs.keySet().toArray();

        for (int i = 0; i < defKeys.length; i++) {
            String[] defVals = (String[]) _defs.get(defKeys[i]);

            if (defVals[0] != null && !_vals.containsKey(defKeys[i])) {
                String[] defaultVal = new String[1];
                defaultVal[0] = defVals[0];
                _vals.put(defKeys[i], defaultVal);
            }
        }
    }
}
