
/*
Copyright (C) Cedilla Software Services Ltd. 1999. License is granted to use this software subject to
the following conditions:-

Preamble:

The intent of this document is to state the conditions under which a Package may be copied, 
such that the Copyright Holder maintains some semblance of artistic control over the development 
of the package, while giving the users of the package the right to use and distribute the 
Package in a more-or-less customary fashion, plus the right to make reasonable modifications.

Definitions:

"Package" refers to the collection of files distributed by the Copyright Holder, and 
derivatives of that collection of files created through textual modification.

"Standard Version" refers to such a Package if it has not been modified, or has been modified 
in accordance with the wishes of the Copyright Holder.

"Copyright Holder" is whoever is named in the copyright or copyrights for the package.

"You" is you, if you're thinking about copying or distributing this Package.

"Reasonable copying fee" is whatever you can justify on the basis of media cost, duplication 
charges, time of people involved, and so on. (You will not be required to  justify it to 
the Copyright Holder, but only to the computing community at large as a market that must bear the fee.)

"Freely Available" means that no fee is charged for the item itself, though there may be 
fees involved in handling the item. It also means that recipients of the item may redistribute 
it under the same conditions they received it.

1. You may make and give away verbatim copies of the source form of the Standard Version of this 
Package without restriction, provided that you duplicate all of the original copyright 
notices and associated disclaimers.

2. You may apply bug fixes, portability fixes and other modifications derived from the Public 
Domain or from the Copyright Holder. A Package modified in such a way shall still be considered 
the Standard Version.

3. You may otherwise modify your copy of this Package in any way, provided that you 
insert a prominent notice in each changed file stating how and when you changed that file, and provided 
that you do at least ONE of the following:

a) place your modifications in the Public Domain or otherwise make them Freely Available, 
such as by posting said modifications to Usenet or an equivalent medium, or placing the 
modifications on a major archive site such as ftp.uu.net, or by allowing the 
Copyright Holder to include your modifications in the Standard Version of the Package.

b) use the modified Package only within your corporation or organization.

c) rename any non-standard executables so the names do not conflict with 
standard executables, which must also be provided, and provide a separate manual page for 
each non-standard executable that clearly documents how it differs from the Standard Version.

d) make other distribution arrangements with the Copyright Holder.

4. You may distribute the programs of this Package in object code or executable 
form, provided that you do at least ONE of the following:

a) distribute a Standard Version of the executables and library files, together with 
instructions (in the manual page or equivalent) on where to get the Standard Version.

b) accompany the distribution with the machine-readable source of the Package 
with your modifications.

c) accompany any non-standard executables with their corresponding Standard Version 
executables, giving the non-standard executables non-standard names, and clearly documenting 
the differences in manual pages (or equivalent), together with instructions on where to 
get the Standard Version.

d) make other distribution arrangements with the Copyright Holder.

5. You may charge a reasonable copying fee for any distribution of this Package. You may 
charge any fee you choose for support of this Package. You may not charge a fee for
this Package itself. However, you may distribute this Package in aggregate with other 
(possibly commercial) programs as part of a larger (possibly commercial) software
distribution provided that you do not advertise this Package as a product of your own.

6. The scripts and library files supplied as input to or produced as output from the 
programs of this Package do not automatically fall under the copyright of this Package, 
but belong to whomever generated them, and may be sold commercially, and may be 
aggregated with this Package.

7. C, Java or perl subroutines supplied by you and linked into this Package shall not be 
considered part of this Package.

8. The name of the Copyright Holder may not be used to endorse or promote products 
derived from this software without specific prior written permission.

9. THIS PACKAGE IS PROVIDED "AS IS" AND WITHOUT ANY EXPRESS OR IMPLIED WARRANTIES, 
INCLUDING, WITHOUT LIMITATION, THE IMPLIED WARRANTIES OF MERCHANTIBILITY AND FITNESS 
FOR A PARTICULAR PURPOSE.

The End
*/
import java.io.*;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.Hashtable;
import java.util.Enumeration;
import com.sun.xml.parser.Resolver;
import com.sun.xml.tree.XmlDocument;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.Text;
import org.xml.sax.InputSource;
import org.xml.sax.SAXException;
import org.xml.sax.SAXParseException;

public class Weave
{
    //Static Members////////////////////////////////////////////////////////////
    private static final String VERSION = "0.5";

    //Member Variables//////////////////////////////////////////////////////////
    private static OptsParser _userOptions;
    private FileOutputStream _fos; 
    private PrintStream _pos = null;
    private Hashtable _classes = new Hashtable();
    private Hashtable _codeFragments = new Hashtable();
    private int _sectionNumber = 1;
    private int _subsectionNumber = 1;
    private String _javadocPrefix;

    //Constructors//////////////////////////////////////////////////////////////
    public Weave(XmlDocument doc) {
        //Respond to javadoc option
        if (_userOptions.containsKey("javadoc")) {
            _javadocPrefix = _userOptions.getOption("javadoc")[0];
            //System.out.println("prefix is " + _javadocPrefix);
        }

        //Respond to version option
        if (_userOptions.containsKey("version")) {
            System.out.println("Weave: version " + VERSION);
        }


	    try {
	        _fos = new FileOutputStream("out.html");
	        _pos = new PrintStream(_fos);
        	    
	    } catch (IOException e) {
	        System.out.println(e);
	    }
	    Element root = doc.getDocumentElement();

        String version = root.getAttribute("VERSION");

        if (version.equals("")) {
            System.out.println("Weave: !!WARNING!! LIT tags need a version attribute");
        }

        Element child = (Element)root.getFirstChild();
        
        outputHeader(root);
        
        if (child.getTagName().equals("COPYRIGHT")) {
            child = (Element)child.getNextSibling();
        }
        
        //Output each section
        while (child != null) {
            processSection(child);
            child = (Element)child.getNextSibling();
        }
        
        outputTrailer();
    }

    //Public Methods////////////////////////////////////////////////////////////
    public static void main (String argv [])
    {
	InputSource	input;
	XmlDocument	doc;

    _userOptions = new OptsParser("Usage: java Weave [-options] source-file");
    _userOptions.addDefinition("javadoc", null, "Optional. This prefixes all javadoc URLS.");
    _userOptions.addDefinition("version", null, "Optional. This prints out the version number.");
    _userOptions.parse(argv);

	if (argv.length < 1) {
	    _userOptions.printAllMessages();
        //_userOptions.dialog("Weave");
	    System.exit (1);
	}

    String fileName = _userOptions.last(1)[0];
    System.out.println("Weave: Processing file " + fileName);

	try {
	    // turn the filename into an input source
	    input = Resolver.createInputSource (new File (fileName));

	    // turn it into an in-memory object
	    doc = XmlDocument.createXmlDocument (input, true);

	    // normalize text representation
	    doc.getDocumentElement ().normalize ();
	    
	    // Weave the document into an HTML file
	    Weave weave = new Weave(doc);

	} catch (SAXParseException err) {
	    System.out.println ("** Parsing error" 
		+ ", line " + err.getLineNumber ()
		+ ", uri " + err.getSystemId ());
	    System.out.println("   " + err.getMessage ());
	    // print stack trace as below

	} catch (SAXException e) {
	    Exception	x = e.getException ();

	    ((x == null) ? e : x).printStackTrace ();

	} catch (Throwable t) {
	    t.printStackTrace ();
	}

	System.exit (0);
    }
    
    private void processSection(Element section) {
        String title = section.getAttribute("TITLE");
        _pos.println("<A NAME = \"SECTION" + _sectionNumber + "\"></A>");
        _pos.println("<H2>" + _sectionNumber + " " + title + "</H2>");
        _subsectionNumber = 1;
        
        Element child = (Element)section.getFirstChild();
        
        while (child != null) {
            if (child.getTagName().equals("NARRATIVE")) {
                processNarrative(child);
            } else if (child.getTagName().equals("CODE")) {
                processCode(child);
            } else if (child.getTagName().equals("CLASS")) {
                processClass(child);
            } else if (child.getTagName().equals("INTERFACE")) {
                processInterface(child);
            } else if (child.getTagName().equals("FILE")) {
                processFile(child);
            } else if (child.getTagName().equals("SUBSECTION")) {
                processSubSection(child);
            }
            child = (Element)child.getNextSibling();
        }
        _sectionNumber++;
    }

    private void processSubSection(Element subsection) {
        String title = subsection.getAttribute("TITLE");
        _pos.println("<A NAME = \"SECTION" + _sectionNumber + "_" + _subsectionNumber + "\"></A>");
        _pos.println("<H3>" + _sectionNumber + "." + _subsectionNumber + " " + title + "</H3>");

        _subsectionNumber++;
        
        Element child = (Element)subsection.getFirstChild();
        
        while (child != null) {
            if (child.getTagName().equals("NARRATIVE")) {
                processNarrative(child);
            } else if (child.getTagName().equals("CODE")) {
                processCode(child);
            } else if (child.getTagName().equals("CLASS")) {
                processClass(child);
            } else if (child.getTagName().equals("INTERFACE")) {
                processInterface(child);
            } else if (child.getTagName().equals("FILE")) {
                processFile(child);
            } 
            child = (Element)child.getNextSibling();
        }
    }
    
    private void outputHeader(Element root) {
        _pos.println("<!-- This document generated automatically from the Marius LP system. Please do not edit.-->");
        _pos.println("<HTML><HEAD>");
        String title = root.getAttribute("TITLE");
        _pos.println("<TITLE>" + title + "</TITLE>");
        _pos.println("</HEAD><BODY><H1>" + title + "</H1>");
    }
    
    private void outputTrailer() {
        _pos.print("</BODY></HTML>");
    }

    private void processNarrative(Element narrative) {
        Element child = (Element)narrative.getFirstChild();
        
        while (child != null) {
            processP(child);
            child = (Element)child.getNextSibling();
        }
    }

    private void processP(Element narrative) {
        Node node = narrative.getFirstChild();
        
        _pos.println("<P>");
        
        while (node != null) {
            if (node instanceof Text) {
                try {
                    Text text = (Text)node;
                    String narString = text.getData();
                    _pos.print(narString);
                } catch (Exception e) {
                    System.out.println(e);
                }
            } else if (((Element)node).getTagName().equals("REF")) { //Node is a ref
                Element child = (Element)node;
                String path = child.getAttribute("PATH");
                Text text = (Text)child.getFirstChild();
                String textString = text.getData();

                _pos.print("<A HREF = \"" + path + "\">" + textString + "</A>");
            } else { //Node is a javadoc
                Element child = (Element)node;
                char[] className = child.getAttribute("CLASS").toCharArray();

                for (int i = 0; i < className.length; i++) {
                    if (className[i] == '.') {
                        className[i] = '/';
                    }
                }

                Text text = (Text)child.getFirstChild();
                String textString = text.getData();

                _pos.print("<A HREF = \"" + _javadocPrefix + new String(className) + ".html\">" + textString + "</A>");

            }
            
            node = node.getNextSibling();
        }
        
        _pos.println("</P>");
    }
    
    private void processCode(Element code) {
        String elide = code.getAttribute("ELIDE");
        
        if (elide.equals("F")) {
            Text text = (Text)code.getFirstChild();
            String name = code.getAttribute("NAME");
            
            try {
                String codeString = text.getData();
                _pos.println("<A NAME = \"" + name + "\"></A>");
                _pos.print("<PRE>");
                
                for (int i = 0; i < codeString.length(); i++) {
                    char c = codeString.charAt(i);
                    
                    if (c == '<') {
                        _pos.print("&lt;");
                    } else {
                        _pos.print(c);
                    }
                }
                _pos.print("</PRE>");
            } catch (Exception e) {
                System.out.println(e);
            }
        }
    }

    private void processFile(Element file) {
        String elide = file.getAttribute("ELIDE");
        
        if (elide.equals("F")) {
            Text text = (Text)file.getFirstChild();
            String name = file.getAttribute("NAME");
            
            try {
                String fileString = text.getData();
                _pos.println("<A NAME = \"" + name + "\"></A>");
                _pos.print("<PRE>");
                
                for (int i = 0; i < fileString.length(); i++) {
                    char c = fileString.charAt(i);
                    
                    if (c == '<') {
                        _pos.print("&lt;");
                    } else {
                        _pos.print(c);
                    }
                }
                _pos.print("</PRE>");
                _pos.println("<A HREF = \"" + name + "\">See the contents of " + name + " here.</A>");
            } catch (Exception e) {
                System.out.println(e);
            }
        }
    }
    
    private void processClass(Element cls) {
        String visibility = cls.getAttribute("VISIBILITY");
        String name = cls.getAttribute("NAME");
        String extendsStr = cls.getAttribute("EXTENDS");
        String implementsStr = cls.getAttribute("IMPLEMENTS");
        String packageStr = cls.getAttribute("PACKAGE");
        boolean abs = cls.getAttribute("ABSTRACT").equals("T");;

        _pos.println("<HR><A NAME = \"" + name + "\"></A><PRE>");
        
        if (packageStr.length() > 0) {
            _pos.println("package " + packageStr + ";");
            _pos.println();
        }
        
        _pos.print(visibility.toLowerCase() + (abs ? " abstract " : " ") + "class " + name + " ");
        
        if (extendsStr.length() > 0) {
            _pos.print("extends " + extendsStr + " ");
        }
        
        if (implementsStr.length() > 0) {
            _pos.print("implements " + implementsStr + " ");
        }
        
        _pos.println(" {");
        _pos.println("    ....");
        _pos.println("}</PRE>");
        _pos.println("<A HREF = \"" + name + ".java\">See the complete code for " + name + " here.</A><HR>");
    }
    
    private void processInterface(Element inter) {
        String visibility = inter.getAttribute("VISIBILITY");        
        String name = inter.getAttribute("NAME");
        String extendsStr = inter.getAttribute("EXTENDS");
        String packageStr = inter.getAttribute("PACKAGE");
        
        _pos.println("<HR><A NAME = \"" + name + "\"></A><PRE>");
        
        if (packageStr.length() > 0) {
            _pos.println("package " + packageStr + ";");
            _pos.println();
        }
        
        _pos.print(visibility.toLowerCase() + " interface " + name + " ");
        
        if (extendsStr.length() > 0) {
            _pos.print("extends " + extendsStr + " ");
        }
                
        _pos.println(" {");
        _pos.println("    ....");
        _pos.println("}</PRE>");
        _pos.println("<A HREF = \"" + name + ".java\">See the complete code for " + name + " here.</A><HR>");
    }    
}
